/***************************************************************************
                          editorpage.h  -  description
                             -------------------
    begin                : Sun May 18 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EDITORPAGE_H
#define EDITORPAGE_H

#include <qwidget.h>
#include <qhbox.h>
#include <qsplitter.h>
#include <qtabwidget.h>
#include <qpopupmenu.h>
#include <ktexteditor/document.h>
#include <ktexteditor/view.h>
#include "ctagsfrontend.h"
#include "ctagslist.h"

/**
 * An editor window based on the system's current editing application.
 * The page is divided into two panes. One holds an embedded editor, and the
 * other holds a list of tags (generated by Ctags) of the file currently being
 * edited.
 * The widget creates an instance of the editor application, and uses its 
 * document and view objects that allow KScope to control it. A page also
 * Each page is inserted in a separate tab in the EditorTabs widget.
 * @author Elad Lahav
 */

class EditorPage : public QHBox
{
   Q_OBJECT

public:
	EditorPage(KTextEditor::Document*, QPopupMenu*, QTabWidget* pParent = 0,
		const char* szName = 0);
	~EditorPage();

	void open(const QString&);
	void setNewFile();
	void save();
	bool close(bool bForce = false);
	void applyPrefs();
	void setEditorFocus();
	
	KTextEditor::Document* getDocument();
	KTextEditor::View* getView();
	bool isNewFile() { return m_bNewFile; }
	QString getFilePath();
	QString getFileName();
	bool isWritable();
	bool isModified();
	QString getSelection();
	QString getWordUnderCursor();
	QString getSuggestedText();
	
	int m_nMenuId;

public slots:
	void slotGotoLine(uint);
	void slotMenuSelect();

signals:
	void fileOpened(EditorPage*, const QString&);
	void newFile(EditorPage*);
	void modified(EditorPage*, bool);
	void cursorPosChanged(uint, uint);
	
private:
	QTabWidget* m_pParentTab;
	CtagsFrontend m_tags;
	QSplitter* m_pSplit;
	CtagsList* m_pCtagsList;
	KTextEditor::Document* m_pDoc;
	KTextEditor::View* m_pView;
	bool m_bOpen;
	bool m_bNewFile;
	
	/** The name of the file being edited. */
	QString m_sName;
	
	/** true if the file system allows this file to be modified, false
	    otherwise. */
	bool m_bWritable;
	
	/** This variable is required in addition to m_pDoc->isModified() so
		that the modified() signal is emitted only once. */
	bool m_bModified;
	
	uint m_nLine;
	
private slots:
	void slotChildResized();
	void slotFileOpened();
	void slotSetModified();
	void slotUndoChanged();
	void slotCursorPosChange();
	void slotCtagsRunning(bool);
};

#endif
