/***************************************************************************
                          kscopeconfig.cpp  -  description
                             -------------------
    begin                : Fri Jun 13 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kconfig.h>
#include <kapplication.h>
#include <kglobalsettings.h>
#include "kscopeconfig.h"

KScopeConfig::ConfParams KScopeConfig::s_cpDef = {
	"/usr/bin/cscope",
	true,
	"/usr/bin/ctags",
	SPLIT_SIZES(),
	QColor(255, 255, 255),
	QColor(0, 0, 0),
	QColor(255, 255, 255),
	QColor(0, 0, 0),
	QColor(255, 255, 255),
	QColor(0, 0, 0),
	QFont(), // Font definitions are overriden in load()
	QFont(),
	QFont(),
	100,
	NameAsc,
	false,
	true,
	true,
	false,
	true,
	"konsole -e vi +%L %F" // External editor example
};

/**
 * Class constructor.
 */
KScopeConfig::KScopeConfig() : m_bFontsChanged(false)
{
}

/**
 * Class destructor.
 */
KScopeConfig::~KScopeConfig()
{
}

/**
 * Reads KScope's parameters from the standard configuration file.
 */
void KScopeConfig::load()
{
	KConfig* pConf = kapp->config();

	// Need a working instance to get the system's default font (cannot be
	// initialised statically)
	s_cpDef.fontFileList = KGlobalSettings::generalFont();
	s_cpDef.fontTagList = KGlobalSettings::generalFont();
	s_cpDef.fontQueryList = KGlobalSettings::generalFont();
	
	// Read the paths to required executables
	pConf->setGroup("Programs");
	m_cp.sCscopePath = pConf->readEntry("CScope");
	m_cp.bVerboseCscope = pConf->readBoolEntry("VerboseCScope",
		s_cpDef.bVerboseCscope);
	m_cp.sCtagsPath = pConf->readEntry("CTags");

	// Read size and position parameters
	pConf->setGroup("Geometry");
	m_cp.siEditor = pConf->readIntListEntry("Editor");
	if (m_cp.siEditor.empty())
		m_cp.siEditor << 200 << 800;
	m_cp.bShowQueryButtons = pConf->readBoolEntry("ShowQueryButtons",
		s_cpDef.bShowQueryButtons);

	// Read the recent projects list
	pConf->setGroup("Projects");
	m_slProjects = pConf->readListEntry("Recent");

	// Read colour settings
	pConf->setGroup("Colors");
	m_cp.clrFileListBack = pConf->readColorEntry("FileListBack",
		&s_cpDef.clrFileListBack);
	m_cp.clrFileListFore = pConf->readColorEntry("FileListFore",
		&s_cpDef.clrFileListFore);
	m_cp.clrTagListBack = pConf->readColorEntry("TagListBack",
		&s_cpDef.clrTagListBack);
	m_cp.clrTagListFore = pConf->readColorEntry("TagListFore",
		&s_cpDef.clrTagListFore);
	m_cp.clrQueryListBack = pConf->readColorEntry("QueryListBack",
		&s_cpDef.clrQueryListBack);
	m_cp.clrQueryListFore = pConf->readColorEntry("QueryListFore",
		&s_cpDef.clrQueryListFore);

	// Read font settings
	pConf->setGroup("Fonts");
	m_cp.fontFileList = pConf->readFontEntry("FileList", 
		&s_cpDef.fontFileList);
	m_cp.fontTagList = pConf->readFontEntry("TagList",
		&s_cpDef.fontTagList);
	m_cp.fontQueryList = pConf->readFontEntry("QueryList",
		&s_cpDef.fontQueryList);
	
	// Other options
	pConf->setGroup("Options");
	m_cp.nMaxPosHistory = pConf->readUnsignedNumEntry("MaxPosHistory", 
		s_cpDef.nMaxPosHistory);
	m_cp.ctagSortOrder = 
		(CtagSort)pConf->readUnsignedNumEntry("CtagSortOrder",
		s_cpDef.ctagSortOrder);
	m_cp.bReadOnlyMode = pConf->readBoolEntry("ReadOnlyMode", 
		s_cpDef.bReadOnlyMode);
	m_cp.bLoadLastProj = pConf->readBoolEntry("LoadLastProj", 
		s_cpDef.bLoadLastProj);
	m_cp.bAutoTagHl = pConf->readBoolEntry("AutoTagHl", 
		s_cpDef.bAutoTagHl);
	m_cp.bBriefQueryCaptions = pConf->readBoolEntry("BriefQueryCaptions", 
		s_cpDef.bBriefQueryCaptions);
	m_cp.sExtEditor = pConf->readEntry("ExternalEditor", s_cpDef.sExtEditor);
}

/**
 * Sets default values to he configuration parameters (except for those where
 * a default value has no meaning, such as the recent projects list).
 */
void KScopeConfig::loadDefault()
{
	m_cp = s_cpDef;
}

/**
 * Loads the layout of the main window.
 * @param	pMainWindow	Pointer to the main docking window
 */
void KScopeConfig::loadWorkspace(KDockMainWindow* pMainWindow)
{
	pMainWindow->readDockConfig(kapp->config(), "Workspace");
}
 
/**
 * Writes KScope's parameters from the standard configuration file.
 */
void KScopeConfig::store()
{
	KConfig* pConf = kapp->config();

	// Write the paths to required executables
	pConf->setGroup("Programs");
	pConf->writeEntry("CScope", m_cp.sCscopePath);
	pConf->writeEntry("VerboseCScope", m_cp.bVerboseCscope);
	pConf->writeEntry("CTags", m_cp.sCtagsPath);

	// Write size and position parameters
	pConf->setGroup("Geometry");
	pConf->writeEntry("Editor", m_cp.siEditor);
	pConf->writeEntry("ShowQueryButtons", m_cp.bShowQueryButtons);

	// Write the recent projects list
	pConf->setGroup("Projects");
	pConf->writeEntry("Recent", m_slProjects);

	// Write colour settings
	pConf->setGroup("Colors");
	pConf->writeEntry("FileListBack", m_cp.clrFileListBack);
	pConf->writeEntry("FileListFore", m_cp.clrFileListFore);
	pConf->writeEntry("TagListBack", m_cp.clrTagListBack);
	pConf->writeEntry("TagListFore", m_cp.clrTagListFore);
	pConf->writeEntry("QueryListBack", m_cp.clrQueryListBack);
	pConf->writeEntry("QueryListFore", m_cp.clrQueryListFore);

	// Write font settings
	if (m_bFontsChanged) {
		pConf->setGroup("Fonts");
		pConf->writeEntry("FileList", m_cp.fontFileList);
		pConf->writeEntry("TagList", m_cp.fontTagList);
		pConf->writeEntry("QueryList", m_cp.fontQueryList);
	}
		
	// Other options
	pConf->setGroup("Options");
	pConf->writeEntry("CtagSortOrder", (uint)m_cp.ctagSortOrder);
	pConf->writeEntry("ReadOnlyMode", m_cp.bReadOnlyMode);
	pConf->writeEntry("LoadLastProj", m_cp.bLoadLastProj);
	pConf->writeEntry("AutoTagHl", m_cp.bAutoTagHl);
	pConf->writeEntry("BriefQueryCaptions", m_cp.bBriefQueryCaptions);
	pConf->writeEntry("ExternalEditor", m_cp.sExtEditor);
	
	// Do not report it's the first time on the next run
	pConf->setGroup("General");
	pConf->writeEntry("FirstTime", false);
}

/**
 * Stores the layout of the main window.
 * @param	pMainWindow	Pointer to the main docking window
 */
void KScopeConfig::storeWorkspace(KDockMainWindow* pMainWindow)
{
	pMainWindow->writeDockConfig(kapp->config(), "Workspace");
}

bool KScopeConfig::isFirstTime()
{
	KConfig* pConf = kapp->config();

	pConf->setGroup("General");
	return pConf->readBoolEntry("FirstTime", true);
}

const QString& KScopeConfig::getCscopePath() const
{
	return m_cp.sCscopePath;
}

void KScopeConfig::setCscopePath(const QString& sPath)
{
	m_cp.sCscopePath = sPath;
}

bool KScopeConfig::useVerboseCscope() const
{
	return m_cp.bVerboseCscope;
}

void KScopeConfig::setVerboseCscope(bool bVerbose)
{
	m_cp.bVerboseCscope = bVerbose;
}

const QString& KScopeConfig::getCtagsPath() const
{
	return m_cp.sCtagsPath;
}

void KScopeConfig::setCtagsPath(const QString& sPath)
{
	m_cp.sCtagsPath = sPath;
}

const QStringList& KScopeConfig::getRecentProjects() const
{
	return m_slProjects;
}

/**
 * Adds the given project path to the beginning of the recently used projects
 * list.
 * @param	sProjPath	The path of the project to add
 */
void KScopeConfig::addRecentProject(const QString& sProjPath)
{
	QStringList::Iterator itr;
	
	itr = m_slProjects.find(sProjPath);
	if (itr != m_slProjects.end())
		m_slProjects.remove(itr);
			
	m_slProjects.prepend(sProjPath);
}

/**
 * Removes the given project path from recently used projects list.
 * @param	sProjPath	The path of the project to remove
 */
void KScopeConfig::removeRecentProject(const QString& sProjPath)
{
	m_slProjects.remove(sProjPath);
}

const SPLIT_SIZES& KScopeConfig::getEditorSizes() const
{
	return m_cp.siEditor;
}

void KScopeConfig::setEditorSizes(const SPLIT_SIZES& siEditor)
{
	m_cp.siEditor = siEditor;
}

const QColor& KScopeConfig::getColor(ConfWidget cw, ColorType type) const
{
	switch (cw) {
	case FileList:
		return (type == Background) ? m_cp.clrFileListBack :
			m_cp.clrFileListFore;

	case TagList:
		return (type == Background) ? m_cp.clrTagListBack :
			m_cp.clrTagListFore;

	case QueryList:
		return (type == Background) ? m_cp.clrQueryListBack :
			m_cp.clrQueryListFore;
	}

	// Default return value, shouldn't happen (but else the compiler may
	// complain)
	return m_cp.clrFileListBack;
}

void KScopeConfig::setColor(ConfWidget cw, ColorType type, const QColor& clr)
{
	switch (cw) {
	case FileList:
		(type == Background) ? m_cp.clrFileListBack = clr :
			m_cp.clrFileListFore = clr;
		break;
		
	case TagList:
		(type == Background) ? m_cp.clrTagListBack = clr :
			m_cp.clrTagListFore = clr;
		break;

	case QueryList:
		(type == Background) ? m_cp.clrQueryListBack = clr :
			m_cp.clrQueryListFore = clr;
		break;
	}

	emit colorChanged(cw, type);
}

const QFont& KScopeConfig::getFont(ConfWidget cw) const
{
	switch (cw) {
	case FileList:
		return m_cp.fontFileList;
		
	case TagList:
		return m_cp.fontTagList;
		
	case QueryList:
		return m_cp.fontQueryList;
	}

	// Default return value, shouldn't happen (but else the compiler may
	// complain)
	return m_cp.fontFileList;
}

void KScopeConfig::setFont(ConfWidget cw, const QFont& font)
{
	switch (cw) {
	case FileList:
		m_cp.fontFileList = font;
		break;
		
	case TagList:
		m_cp.fontTagList = font;
		break;

	case QueryList:
		m_cp.fontQueryList = font;
		break;
	}

	m_bFontsChanged = true;
	emit fontChanged(cw);
}

/**
 * @return	The maximal size of the positions history list
 */
uint KScopeConfig::getMaxPosHistory()
{
	return m_cp.nMaxPosHistory;
}

KScopeConfig::CtagSort KScopeConfig::getCtagSortOrder()
{
	return m_cp.ctagSortOrder;
}

void KScopeConfig::setCtagSortOrder(CtagSort ctagSortOrder)
{
	m_cp.ctagSortOrder = ctagSortOrder;
}

bool KScopeConfig::getReadOnlyMode()
{
	return m_cp.bReadOnlyMode;
}

void KScopeConfig::setReadOnlyMode(bool bReadOnlyMode)
{
	m_cp.bReadOnlyMode = bReadOnlyMode;
}

bool KScopeConfig::getLoadLastProj()
{
	return m_cp.bLoadLastProj;
}

void KScopeConfig::setLoadLastProj(bool bLoadLastProj)
{
	m_cp.bLoadLastProj = bLoadLastProj;
}

bool KScopeConfig::getAutoTagHl()
{
	return m_cp.bAutoTagHl;
}

void KScopeConfig::setAutoTagHl(bool bAutoTagHl)
{
	m_cp.bAutoTagHl = bAutoTagHl;
}

bool KScopeConfig::getUseBriefQueryCaptions()
{
	return m_cp.bBriefQueryCaptions;
}

void KScopeConfig::setUseBriefQueryCaptions(bool bBrief)
{
	m_cp.bBriefQueryCaptions = bBrief;
}

bool KScopeConfig::getShowQueryButtons()
{
	return m_cp.bShowQueryButtons;
}

void KScopeConfig::setShowQueryButtons(bool bShow)
{
	m_cp.bShowQueryButtons = bShow;
}

const QString& KScopeConfig::getExtEditor()
{
	return m_cp.sExtEditor;
}

void KScopeConfig::setExtEditor(const QString& sExtEditor)
{
	m_cp.sExtEditor = sExtEditor;
}

/**
 * Determines if an external editor should be used.
 * An external editor is used if KScope is in Read-Only mode, and a command-
 * line for the editor was specified.
 * @return	true to use an external editor, false otherwise
 */
bool KScopeConfig::useExtEditor()
{
	bool b = (m_cp.bReadOnlyMode && !m_cp.sExtEditor.isEmpty());
	return b;
}

/**
 * Returns a reference to a global configuration object.
 * The static object defined is this function should be the only KSCopeConfig
 * object in this programme. Any code that wishes to get or set global
 * configuration parameters, should call Config(), instead of defining its
 * own object.
 * @return	Reference to a statically allocated configuration object
 */
KScopeConfig& Config()
{
	static KScopeConfig conf;
	return conf;
}
