/***************************************************************************
                          openprojectdlg.cpp  -  description
                             -------------------
    begin                : Wed Jun 11 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad@eldarshany.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <kconfig.h>
#include <kfiledialog.h>
#include "openprojectdlg.h"
#include "kscopeconfig.h"

/**
 * Class constructor.
 * @param	pParent	The parent widget
 * @param	szName	The widget's name
 */
OpenProjectDlg::OpenProjectDlg(QWidget* pParent, const char* szName) :
	OpenProjectLayout(pParent, szName)
{
	loadRecent();
	
	connect(m_pOpenButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(m_pCancelButton, SIGNAL(clicked()), this, SLOT(reject()));

	// Browse for a project when the "Find..." button is clicked
	connect(m_pFindButton, SIGNAL(clicked()), this, SLOT(slotFindProject()));

	// Remove a project from the recent projects list when the "Remove"
	// button is clicked
	connect(m_pRemoveButton, SIGNAL(clicked()), this,
		SLOT(slotRemoveProject()));

	// Set the selected project when its path is clicked in the recent
	// projects list
	connect(m_pRecentList, SIGNAL(clicked(QListBoxItem*)), this,
		SLOT(slotSelectProject(QListBoxItem*)));

	// Open the project when its path is double-clicked in the recent
	// projects list
	connect(m_pRecentList, SIGNAL(doubleClicked(QListBoxItem*)), this,
		SLOT(slotOpenProject(QListBoxItem*)));
}

/**
 * Class destructor.
 */
OpenProjectDlg::~OpenProjectDlg()
{
}

void OpenProjectDlg::loadRecent()
{
	const QStringList& slProjects = Config().getRecentProjects();
	QStringList::const_iterator itr;

	for (itr = slProjects.begin(); itr != slProjects.end(); ++itr)
		m_pRecentList->insertItem(*itr);
}

void OpenProjectDlg::slotFindProject()
{
	QString sPath;
	
	sPath = KFileDialog::getOpenFileName(QString::null, "cscope.proj");
	if (sPath.isEmpty())
		return;

	QFileInfo fi(sPath);
	m_pPathEdit->setText(fi.dirPath(true));
}

/**
 * Removes a project from the recent projects list.
 */
void OpenProjectDlg::slotRemoveProject()
{
	QListBoxItem* pItem;

	// Remove the selected item, if any
	pItem = m_pRecentList->selectedItem();
	if (pItem != NULL) {
		Config().removeRecentProject(pItem->text());
		m_pRecentList->removeItem(m_pRecentList->currentItem());
	}
}

void OpenProjectDlg::slotSelectProject(QListBoxItem* pItem)
{
	if (pItem != NULL)
		m_pPathEdit->setText(pItem->text());
}

void OpenProjectDlg::slotOpenProject(QListBoxItem* pItem)
{
	if (pItem != NULL) {
		m_pPathEdit->setText(pItem->text());
		accept();
	}
}
