/***************************************************************************
                          querypage.h  -  description
                             -------------------
    begin                : Sat May 24 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad@eldarshany.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QUERYPAGE_H
#define QUERYPAGE_H

#include <qwidget.h>
#include <qhbox.h>
#include <qlistview.h>
#include "frontend.h"

/**
 * Defines a single page in the query widget.
 * Each page displays a list of query result records. When a result record is
 * selected, a signal is emitted notifying the main widget it should display
 * the source file, and select the line, in this record.
 * @author Elad Lahav
 */

class QueryPage : public QHBox, public FrontendDataTarget
{
   Q_OBJECT

public: 
	QueryPage(QWidget* pParent = 0, const char* szName = 0);
	~QueryPage();
	
	virtual void dataReady(FrontendToken*);
	virtual void finished(int);
	void applyPrefs();
	bool load(const QString&, const QString&);
	QString store(const QString&);
	void clear();
	
	QListView* getList() { return m_pList; }
	void setLocked(bool bLocked) { m_bLocked = bLocked; }
	bool isLocked() { return m_bLocked; }
	void setQueryData(uint, const QString&);
	uint getQueryType() const { return m_nType; }
	const QString& getQueryText() const { return m_sText; }
	QString getQueryCaption(bool bBrief = false) const;

signals:
	void recordSelected(QListViewItem*);
	void queryFinished();
	
private:
	QListView* m_pList;
	bool m_bLocked;
	uint m_nType;
	QString m_sText;
	QString m_sName;
};

#endif
